<?php
/**
 * Main application class implementation.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Main application class, loads config, inits router and front controller
 * and via run method dispatches request.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class HomeWebApp
{
 /** Main configuration section in the XML config file
  */
 const MAIN_CONFIG_SECTION="homePage";

 /** 
  * Enable/disable debug settings
  */
 const DEBUG=true;

 /** The application XML configuration file name, file must be located in the
  * {@link HomeWebApp:$_mainAppDir} directory.
  */
 const CONFIG_FILENAME="config.xml";

 /** directory path to the Zend Framework library.
  * 
  * It must be a full path. Leave empty if the "Zend" directory from the 
  * zend framework is located in the "lib" directory.   
  *   
  * @var string
  */
 //const ZF_DIR="/data/web/virtuals/mirdaweb.cz/app/lib";
 const ZF_DIR="C:/Program Files/Apache Group/Apache2/htdocs/zendframework/library";

 /** Singleton instance
  * @var HomeWebApp
  */
 private static $_instance = null;

 /** application configuration
  *
  * This is the generic object with this public member variables:
  * - mainDir - main application directory
  * - modulesDir - directory for application modules
  * - defaultDir - directory for default module
  * - defaultControllerDir - directory for default module controllers
  * - defaultViewDir - directory for default module views
  * - blogDir - directory for blog module
  * - blogControllerDir - directory for blog module controllers
  * - blogViewDir- directory for blog module views
  * - logDir - directory for application logs if some exist
  * @var object
  */
 private $_config;

 /** application router
  * @var Zend_Controller_Router_Interface
  */
 private $_router;

 /** application front controller
  * @var Zend_Controller_Front
  */
 private $_ctrl;
 
 /** Full main application directoty path.
  * @var string
  */
 private $_mainAppDir;

 /**
  * application locale for
  * @var Zend_Locale
  */
 private $_locale;

 /**
  * application translator object - for I18N
  * @var Zend_Translate
  */
 private $_translator;

 /**
  * list of all available languages, it is set from the $_availableLocales
  * in the constructor
  * @var array
  */
 private $_availableLanguages;

 /**
  * list of all available locales
  * @var array
  */
 private $_availableLocales=array("en_US","cs_CZ");
 
//==============================================================================
/**
 * Singleton pattern implementation makes "new" unavailable
 * @return void
 */
private function __construct($appDir)
{
 //set timezone for date and time functions
 date_default_timezone_set('Europe/Prague');

 if (empty($appDir)) {
  $this->setMainAppDir( realpath(dirname(__FILE__)) );
 } else {
  $this->setMainAppDir($appDir);
 }

 //library directory - constructed from a real path
 $libDir=$this->_mainAppDir.DIRECTORY_SEPARATOR."lib";
 //IncludePath wrapper
 require_once $libDir.DIRECTORY_SEPARATOR."IncludePath.php";
 //add lib and onto include path
 IncludePath::add($libDir);
 //add main app onto include path
 IncludePath::add($this->_mainAppDir);
 //add Zend Framework dir to the include path
 IncludePath::add(self::ZF_DIR);

 require_once "Zend".DIRECTORY_SEPARATOR."Loader.php";
 //register __autoload for the Zend_Loader
 Zend_Loader::registerAutoload();

 //sets available languages
 foreach($this->_availableLocales as $localeStr) {
  $locale=new Zend_Locale($localeStr);
  $this->_availableLanguages[]=$locale->getLanguage();
 }
}

//==============================================================================
/**
 * Singleton pattern implementation makes "clone" unavailable
 * @return void
 */
private function __clone()
{}

//==============================================================================
/**
 * Sets a locale and language by language string.
 * The language string must one of {@link $_availableLanguages} array values.
 * @return void
 * @throws Exception if requested language/locale isn't available
 */
public function setLang($langStr)
{
  switch ($langStr) {
   case "en":
    $this->setLocale("en_US");
    break;
   case "cs":
    $this->setLocale("cs_CZ");
    break;
   default:
    throw new Exception("Cannot switch to language $langStr.");
  }
  
  //set tranlation data
  $lang=$this->_locale->getLanguage();
  $this->_translator->addTranslation($this->getTranslatedFile($lang),$lang,array('clear' => true));
  //necessary
  $this->_translator->setLocale($lang);
}  

//==============================================================================
public function setLocale($localeStr=null)
{
 $session = new Zend_Session_Namespace();

 $findLocale=null;
 
 try {
  if (is_null($localeStr)) {
   //if locale is in a session, take it, set it
   if (isset($session->locale)) {
    $findLocale=new Zend_Locale($session->locale);
   } else {
    $findLocale=new Zend_Locale();
   }
  } else {
   $findLocale=new Zend_Locale($localeStr);
  }
  //when locale autodetection fails, sets en_Us locale
 } catch (Zend_Locale_Exception $e) {
  $findLocale=new Zend_Locale("en_Us");
 }

 //if automatocally detect locale isn't in available locales
 //sets locale to en_us
 if (!in_array($findLocale->toString(),$this->_availableLocales)) {
  $findLocale->setLocale("en_US");
 }

 $this->_locale=$findLocale;
 //save locale identifier into session
 $session->locale=$this->_locale->toString(); 
}

//==============================================================================
public function getLocale()
{
 return $this->_locale; 
}

//==============================================================================
public function getTranslate()
{
 return $this->_translator;
}

//==============================================================================
public function getTranslatedFile($lang)
{
 return $this->_mainAppDir.DIRECTORY_SEPARATOR. "languages"
  .DIRECTORY_SEPARATOR
  ."messages.$lang.mo";
}

//==============================================================================
public function getAvailableLanguages()
{
 return $this->_availableLanguages; 
}

//==============================================================================
public static function getInstance()
{
 if (null === self::$_instance) {
   throw new Exception("The application isn't created, call createApp() first.");
 }
 return self::$_instance;
}  
 
//==============================================================================
/**
 * Singleton pattern implementation.
 * Only one instance for a whole system is allowed.
 * @param string $appDir application directory, if is empty
 * current directory with this script is set as the application directory
 * @return HomeWebApp
 */
public static function createApp($appDir="")
{
 if (null === self::$_instance) {
  self::$_instance = new self($appDir);
 } else {
  throw new Exception("Application is already created");
 } 

 self::$_instance->_loadConfig();
 self::$_instance->_initRouter();
 self::$_instance->_initFrontController();

//set locale
 self::$_instance->setLocale();
 
 //set translation
 $lang=self::$_instance->_locale->getLanguage();
 self::$_instance->_translator=new Zend_Translate('gettext',self::$_instance->getTranslatedFile($lang),$lang);

 return self::$_instance;
}

//==============================================================================
/**
 * Setter for the main application direcotry
 * @return void
 */
public function setMainAppDir($dir)
{
 $this->_mainAppDir=$dir;
}

//==============================================================================
/**
 * Loads the main configuration from the XML config and set configuration -
 * $_config. Sets include_path for some necessary directories.
 * @throws Exception when $_mainAppDir is not set
 */
private function _loadConfig()
{
 if (!isset($this->_mainAppDir))
  throw new Exception("cannot load the application configuration, main application
   directory isn't set");

 //load the application configuration
 $confObj = new Zend_Config_Xml($this->_mainAppDir.DIRECTORY_SEPARATOR.
    self::CONFIG_FILENAME,self::MAIN_CONFIG_SECTION);
 //convert to array
 $confArr=$confObj->toArray();

 //destroy config object
 $confObj = null;

 //sets info about directories 
 $confArr['mainDir'] = $this->_mainAppDir;
 $confArr['modulesDir'] = $this->_mainAppDir . DIRECTORY_SEPARATOR . "modules";

 $confArr['defaultDir'] = $confArr['modulesDir'] . DIRECTORY_SEPARATOR . "default";
 $confArr['defaultControllerDir'] = $confArr['defaultDir'] . DIRECTORY_SEPARATOR
  . "controllers";
 $confArr['defaultViewDir'] = $confArr['defaultDir'] . DIRECTORY_SEPARATOR . "views";

 $confArr['blogDir'] = $confArr['modulesDir'] . DIRECTORY_SEPARATOR . "blog";
 $confArr['blogControllerDir'] = $confArr['blogDir'] . DIRECTORY_SEPARATOR . "controllers";
 $confArr['blogModelDir'] = $confArr['blogDir'] . DIRECTORY_SEPARATOR . "models";
 $confArr['blogViewDir'] = $confArr['blogDir'] . DIRECTORY_SEPARATOR . "views";

 $confArr['logDir'] = $this->_mainAppDir . DIRECTORY_SEPARATOR . "logs";

 //$this->_config =(object)$confArr;
 $this->_config=Utils::recursiveToObjectCast($confArr);

 //destroy temporary array
 $confArr = null;

 //save config to registry as object
 Zend_Registry::set("conf",$this->_config);

 //add path for the model directories
 IncludePath::add($this->_config->blogModelDir);
}

//==============================================================================
/**
 * Inits application router and sets necessary routers for application's action
 * controllers.
 */
private function _initRouter()
{
 //create router
 $this->_router = new Zend_Controller_Router_Rewrite();

/*
- define routes, more specific routes must be defined last, routes are matched
  in the reverse order
- set defaults for all used parameters in Zend_Controller_Router_Route is necessary!
- controller names in the 'controller' item in the second parametr of Zend_Controller_Router_Route
  contructor are reformated to the lowecase form, then first char is uppercased added "Controller"
  name and this file is used as action class for this route
  examples:
  'controller' => 'ndevicehwtype' => file(class) name NdevicehwtypeController
  'controller' => 'nDeviceHwType' => file(class) name NdevicehwtypeController
*/

 //default module index controller route
 $this->_router->addRoute('mainroute',
     new Zend_Controller_Router_Route(':action',
     array('controller' =>'index','action'=>'index'))
  );

 //default module project controller route
 $this->_router->addRoute('projectsroute',
     new Zend_Controller_Router_Route('projects/:action/:name',
     array('controller'=>'project','action'=>'index','name'=>null))
 );

 //blog module index controller route
 $this->_router->addRoute('blog',
     new Zend_Controller_Router_Route('blog/:entry',
     array('module' =>'blog','controller'=>'index','action'=>'index','entry'=>null))
 );

 //blog module category content route
 $this->_router->addRoute('blogCategory',
     new Zend_Controller_Router_Route('blog/category/:name',
     array('module' =>'blog','controller'=>'index','action'=>'viewCategory','name'=>null))
 );

 //blog module archive route
 $this->_router->addRoute('blogArchive',
     new Zend_Controller_Router_Route('blog/archive/:year/:month',
     array('module' =>'blog','controller'=>'index','action'=>'viewArchive'))
 );

 //blog module feed route
 $this->_router->addRoute('blogFeed',
     new Zend_Controller_Router_Route('blog/feed',
     array('module' =>'blog','controller'=>'index','action'=>'feed'))
 );

 //blog module comment feed route
 $this->_router->addRoute('blogCommentFeed',
     new Zend_Controller_Router_Route('blog/commentFeed',
     array('module' =>'blog','controller'=>'index','action'=>'commentFeed'))
 );

 //blog module searchResult route
 $this->_router->addRoute('blogSearchResult',
     new Zend_Controller_Router_Route('blog/searchResult',
     array('module' =>'blog','controller'=>'index','action'=>'searchResult'))
 );

 //blog module admin controller route
 $this->_router->addRoute('blogadmin',
     new Zend_Controller_Router_Route('blog/admin/:action/:itemId',
     array('module' =>'blog','controller'=>'admin','action'=>null,'itemId'=>null))
 );

}

//==============================================================================
/**
 * Initializes application's front controller.
 * @throws Exception when {@link HomeWebApp::$_router router} or {@link
 * HomeWebApp::$_config config} isn't correctly initialized.
 */
private function _initFrontController()
{
 if (!isset($this->_router) && !isset($this->_config))
  throw new Exception("cannot initialize the front controller, router or config
   aren't instantiated.");
  
 //create Front Controller
 $this->_ctrl = Zend_Controller_Front::getInstance();

 //throw exception from the front controller only for debugging
 $this->_ctrl->throwExceptions(self::DEBUG);
 
 //enable view action helper from helper broker, not necessary, because it is default
 //but it is more explanatory
 $this->_ctrl->setParam('noViewRenderer', false);

 //add special prefix for view helpers and filters in the ViewRenderer
 //action plugin
 $viewRenderer=Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
 $viewRenderer->initView(null,"HomeWebApp_View");
 //set application url enumerator available in a view
 $viewRenderer->view->appUrlEnum=AppUrlEnum::getInstance();

 //set router in a controller
 $this->_ctrl->setRouter($this->_router);

 //set controllers directory
 //$this->_ctrl->setControllerDirectory($this->_config->controllerDir);
 $this->_ctrl->addModuleDirectory($this->_config->modulesDir);

 //build a request object as http request
 $request=new Zend_Controller_Request_Http();
 //autodetect baseurl for the request
 $request->setBaseUrl(null);
 //set request in controller
 $this->_ctrl->setRequest($request);

 //register plugin for the site template
 $this->_ctrl->registerPlugin(new SiteTemplatePlugin());
}

//==============================================================================
/**
 * Runs our application.
 * Just dispatches requests into an appropriate controller actions.
 * @throws Exception when the {@link HomeWebApp::$_ctrl controller} isn't set.
 */
public function run()
{
 //remove slashes from automatically quotes " ' \ etc which are quoted by 
 //get_magic_quotes_gpc(), code (c) Jakub Vrana 
 if (get_magic_quotes_gpc()) {
  $process = array(&$_GET, &$_POST, &$_COOKIE); //maybe add  &$_REQUEST, &$_FILES
  while (list($key, $val) = each($process)) {
   foreach ($val as $k => $v) {
    unset($process[$key][$k]);
    if (is_array($v)) {
     $process[$key][stripslashes($k)] = $v;
     $process[] = &$process[$key][stripslashes($k)];
    } else {
     $process[$key][stripslashes($k)] = stripslashes($v);
    }
   }
  }
  unset($process);
 }

 if (!isset($this->_ctrl))
  throw new Exception("cannot run the application, front controller
   isn't instantiated.");

 //call dispatcher, dispatch requests into the controller action
 $this->_ctrl->dispatch();
}

#END CLASS
}
