<?php
/**
 * Article Dao
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Article Dao class based on the Zend_Db_Table  
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007-2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class Dao_Article extends Dao
{
 /** table name */
 const TABLE_NAME = "article";
 
 /** table name
  * @var string
  */
 protected $_name = self::TABLE_NAME;

 /** primary key column
  * @var string
  */
 protected $_primary = "id";

 /** reference map for the relationship
  * @var array
  */
 protected $_referenceMap = array(
  'Author' => array(
    'columns' => 'author_id',
    'refTableClass' => 'Dao_Author',
    'refColumns' => 'id'
  )
 );

//==============================================================================
/**
 * Get all articles, with some extended information like comments count, author
 * name etc. Has the pagination support.
 *
 * @param int $pageNo current page number, first page has number 1
 * @param int $perPageItems number of items per page
 * @param array $extendInfo what extend information we want, supported keys are
 *  - comment : for the comment count
 *  - author : for the author username
 * @return array the sequential array of rows with article properties
 */
public function fetchAllItems($pageNo=null,$perPageItems=null,$extendInfo=null)
{
 $commentT=Dao_Comment::TABLE_NAME;
 $authorT=Dao_Author::TABLE_NAME;

 $select=$this->getAdapter()->select()
   ->from($this->_name)
   ->order("$this->_name.posted desc");

 if (is_array($extendInfo)) {
  $select->group("$this->_name.id");
  if (in_array("comment",$extendInfo))
    $select->joinLeft($commentT,"$commentT.article_id = $this->_name.id",array("commentCount"=>"count($commentT.id)"));
  if (in_array("author",$extendInfo))
   $select->joinLeft($authorT,"$authorT.id = $this->_name.author_id",array("authorName"=>"username"));
 }

 self::$_dataSource->addSQLlimit($select,$pageNo,$perPageItems);

 return $select->query()->fetchAll();
}

//==============================================================================
/**
 * Returns month statistic for the articles.
 * @return array result table as an array with columns
 *  - article counts
 *  - month name
 *  - year
 */
public function getMonthStat()
{
 $select=$this->getAdapter()->select()
   ->from($this->_name,
     array(
      "artcnt"=>"count(*)",
      "yearMonth"=>"EXTRACT(YEAR_MONTH FROM posted)",
      "year"=>"YEAR(posted)",
      "month"=>"MONTH(posted)"
     ))
   ->group("yearMonth")
   ->order("year desc")
   ->order("month desc");

 return $select->query()->fetchAll();
}

//==============================================================================
/**
 * Gets count of articles for the specific month in the specific year.
 * @param int $year
 * @param int $month
 * @returns int
 */
public function getMonthItemsCount($year,$month)
{
 $sql="SELECT count(*)"
  . "FROM  $this->_name "
  . "WHERE YEAR(posted)=$year AND MONTH(posted)=$month";

 return $this->getAdapter()->fetchOne($sql);
}

//==============================================================================
/**
 * Returns article list for the specific month in the specific year.
 * Just proxy for the dao method call. A pagination is supported.
 * @param int $year
 * @param int $month
 * @param int $pageNo current page number, firts page has no. 1
 * @param int $perPageItems items per page
 * @return array result table as array
 */
public function getMonthItems($year,$month,$pageNo=null,$perPageItems=null)
{
 $sql="SELECT * "
  . "FROM $this->_name "
  . "WHERE YEAR(posted)=$year AND MONTH(posted)=$month "
  . "ORDER BY posted desc";

 $sql=self::$_dataSource->addSQLlimit($sql,$pageNo,$perPageItems);

 return $this->getAdapter()->fetchAll($sql);
}

//==============================================================================
/**
 * Finds article by a title url.
 * @param string $titleUrl
 * @return Zend_Db_Table_Row_Abstract or null
 * @throws new Zend_Db_Exception when some database operation fails
 */
public function findByTitleUrl($titleUrl)
{
 $whereSQL=$this->getAdapter()->quoteInto('titleURL = ?',$titleUrl);
 return $this->fetchRow($whereSQL);
}

#END CLASS
}
